@@ -43,6 +43,7 @@
 #include "tsh-status-dialog.h"
 #include "tsh-log-message-dialog.h"
 #include "tsh-log-dialog.h"
+#include "tsh-blame-dialog.h"
 
 #include "tsh-common.h"
 
@@ -444,6 +445,12 @@ tsh_check_cancel(void *baton)
 	return SVN_NO_ERROR;
 }
 
+void
+tsh_reset_cancel()
+{
+  cancelled = FALSE;
+}
+
 static const gchar *
 tsh_action_to_string(svn_wc_notify_action_t action)
 {
@@ -542,7 +549,7 @@ tsh_notify_state_to_string(enum svn_wc_notify_state_t state)
   return _(state_string);
 }
 
-static const gchar *
+const gchar *
 tsh_status_to_string(enum svn_wc_status_kind status)
 {
   static const gchar const * const status_table[] = {
@@ -757,6 +764,28 @@ tsh_log_func (void *baton, apr_hash_t *changed_paths, svn_revnum_t revision, con
 	return SVN_NO_ERROR;
 }
 
+svn_error_t *
+tsh_blame_func (void *baton, apr_int64_t line_no, svn_revnum_t revision, const char *author, const char *date, const char *line, apr_pool_t *pool)
+{
+  apr_time_t date_val;
+  gchar *date_str = NULL;
+	TshBlameDialog *dialog = TSH_BLAME_DIALOG (baton);
+
+  if(date)
+  {
+    svn_time_from_cstring(&date_val, date, pool);
+    apr_ctime((date_str = g_new0(gchar, APR_CTIME_LEN)), date_val);
+  }
+
+  gdk_threads_enter();
+  tsh_blame_dialog_add(dialog, line_no, revision, author, date_str, line);
+  gdk_threads_leave();
+
+  g_free(date_str);
+
+	return SVN_NO_ERROR;
+}
+
 gchar *
 tsh_strerror(svn_error_t *err)
 {
